/* $Id: poseidon.h,v 1.1 1999/09/14 01:21:01 sjhill Exp $
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 */

#ifndef __POSEIDON_H__
#define __POSEIDON_H__

#include <asm/addrspace.h>
 
/******************************************************************************
*
* 	01  General macro definitions
*
******************************************************************************/

#define REGISTER_BASE   KSEG1ADDR(0x10c00000)

#ifndef ASSEMBLY

	#define REG_AT(x)	(*((volatile unsigned long *)(REGISTER_BASE + x)))

#else

	#define REG_AT(x)   (REGISTER_BASE + x)

#endif

#define BIT(x)	(1 << x)

/******************************************************************************
*
* 	02  Bus Interface Unit
*
******************************************************************************/

#define MemConfig0    REG_AT(0x000)
#define MemConfig1    REG_AT(0x004)
#define MemConfig2    REG_AT(0x008)
#define MemConfig3    REG_AT(0x00c)
#define MemConfig4    REG_AT(0x010)
#define MemConfig5    REG_AT(0x014)
#define MemConfig6    REG_AT(0x018)
#define MemConfig7    REG_AT(0x01c)
#define MemConfig8    REG_AT(0x020)

/* Memory config register 1 */

#define MEM1_ENCS1USER	BIT(21)

/* Memory config register 3 */

#define MEM3_CARD1ACCVAL_MASK	(BIT(24) | BIT(25) | BIT(26) | BIT(27))
#define MEM3_CARD1IOEN		BIT(4)

/* Memory config register 4 */

#define MEM4_ARBITRATIONEN	BIT(29)
#define MEM4_MEMPOWERDOWN	BIT(16)
#define MEM4_ENREFRESH1		BIT(15)
#define MEM4_ENREFRESH0		BIT(14)


/******************************************************************************
*
*	06  UART module
*
******************************************************************************/

#define UartA_Ctrl1   REG_AT(0x0b0)
#define UartA_Ctrl2   REG_AT(0x0b4)
#define UartA_DMActl1 REG_AT(0x0b8)
#define UartA_DMActl2 REG_AT(0x0bc)
#define UartA_DMAcnt  REG_AT(0x0c0)
#define UartA_Data    REG_AT(0x0c4)
#define UartB_Ctrl1   REG_AT(0x0c8)
#define UartB_Ctrl2   REG_AT(0x0cc)
#define UartB_DMActl1 REG_AT(0x0d0)
#define UartB_DMActl2 REG_AT(0x0d4)
#define UartB_DMAcnt  REG_AT(0x0d8)
#define UartB_Data    REG_AT(0x0dc)

/* bit allocations within UART control register 1 */
 
#define UART_ON				BIT(31)	/* indicates status of UART */
#define UART_TX_EMPTY		BIT(30)	/* tx holding and shift registers empty */
#define UART_PRX_HOLD_FULL	BIT(29)	/* pre-rx holding register full */
#define UART_RX_HOLD_FULL	BIT(28)	/* rx holding register is full */
#define UART_EN_DMA_RX 		BIT(15)	/* enable rx DMA */
#define UART_EN_DMA_TX 		BIT(14)	/* enable tx DMA */
#define UART_BREAK_HALT 	BIT(12)	/* enable halt after receiving break */
#define UART_DMA_LOOP		BIT(10)	/* enable DMA loop-roud */
#define UART_PULSE_THREE	BIT(9)	/* tx data is 3 low pulses */
#define UART_PULSE_SIX		BIT(8)	/* tx data is 6 low pulses */
#define UART_DT_INVERT		BIT(7)	/* invert txd and rxd */
#define UART_DIS_TXD		BIT(6)	/* set txd low */
#define UART_LOOPBACK		BIT(4)	/* enable loopback mode */
#define UART_ENABLE		BIT(0)	/* enable UART */
		 
#define SER_SEVEN_BIT		BIT(3)	/* use 7-bit data */
#define SER_EIGHT_BIT		    0	/* use 8-bit data */
#define SER_EVEN_PARITY 	(BIT(2) | BIT(1))	/* use even parity */
#define SER_ODD_PARITY  	BIT(1)	/* use odd parity */
#define SER_NO_PARITY		    0	/* enable parity checking */
#define SER_TWO_STOP		BIT(5)	/* transmit 2 stop bits */
#define SER_ONE_STOP		    0	/* transmit 1 stop bits */

#define SER_SOFT_FLOW		BIT(4)	/* XON/XOFF flow control */
#define SER_HARD_FLOW		BIT(6)	/* CTS/DTR flow control */
#define SER_USE_RTS		BIT(7)	/* configures output DTR as RTS instead */
#define SER_USE_DSR		BIT(8)	/* configures input CTS as DSR instead */

#define SER_NO_FLOW		0

#define UART_FORMAT_MASK	(BIT(1) | BIT(2) | BIT(3) | BIT(5))
#define UART_FLOW_MASK		(BIT(6) | BIT(7) | BIT(8))

/* Baud rate definitions for UART control register 2 */

#define SER_BAUD_38400     ( 5 << 16)		/* divisors are  3.6864MHz      */
#define SER_BAUD_19200     (11 << 16)		/*		----------- - 1 */
#define SER_BAUD_9600      (22 << 16)		/*		(baud * 16)     */
#define SER_BAUD_4800      (47 << 16)
#define SER_BAUD_2400      (95 << 16)		/* divisors are put in the upper     */
#define SER_BAUD_1200      (191 << 16)		/* byte so that they can be combined */
#define SER_BAUD_600       (383 << 16)		/* with the parity and data bit      */
#define SER_BAUD_300	   (767 << 16)		/* constants defined above 			 */



/******************************************************************************
*
*	07  Interrupt module
*
******************************************************************************/

/* Register locations */

#define IntStatus1    REG_AT(0x100)
#define IntStatus2    REG_AT(0x104)
#define IntStatus3    REG_AT(0x108)
#define IntStatus4    REG_AT(0x10c)
#define IntStatus5    REG_AT(0x110)
#define IntStatus6    REG_AT(0x114)

#define IntClear1     REG_AT(0x100)
#define IntClear2     REG_AT(0x104)
#define IntClear3     REG_AT(0x108)
#define IntClear4     REG_AT(0x10c)
#define IntClear5     REG_AT(0x110)
#define IntClear6     REG_AT(0x114)

#define IntEnable1    REG_AT(0x118)
#define IntEnable2    REG_AT(0x11c)
#define IntEnable3    REG_AT(0x120)
#define IntEnable4    REG_AT(0x124)
#define IntEnable5    REG_AT(0x128)
#define IntEnable6    REG_AT(0x12c)

/* CP0 Status Register (sr) */

#define CP0SR_ENABLE_CP0	BIT(28)  /* Cliff changed from 31 */
#define CP0SR_VECTOR		BIT(22)
#define CP0SR_IEc			BIT(0)
#define CP0SR_KERNEL		(BIT(5) | BIT(3) | BIT(1))

#define CP0SR_SOFT_INT0	BIT(8)
#define CP0SR_SOFT_INT1	BIT(9)
#define CP0SR_HARD_INT0	BIT(10)
#define CP0SR_HARD_INT1	BIT(11)		
#define CP0SR_HARD_INT2	BIT(12)			/* connected to IRQ LOW */
#define CP0SR_HARD_INT3	BIT(13)
#define CP0SR_HARD_INT4	BIT(14)			/* connected to IRQ HIGH */
#define CP0SR_HARD_INT5	BIT(15)

#define CP0SR_SOFT_INT_MASK (BIT(8) | BIT(9))
#define CP0SR_HARD_INT_MASK (BIT(10) | BIT(11) | BIT(12) | BIT(13) | BIT(14) | BIT(15))

#define CP0SR_INT_BITS	(CP0SR_SOFT_INT_MASK | CP0SR_HARD_INT_MASK)
#define CP0SR_INT_MASK	~(CP0SR_SOFT_INT_MASK | CP0SR_HARD_INT_MASK)

#ifdef DEVELOPMENT
#define CP0SR_DISABLE_INTS	(CP0SR_ENABLE_CP0 | CP0SR_IEc)
#else
#define CP0SR_DISABLE_INTS	(CP0SR_ENABLE_CP0 | CP0SR_IEc | CP0SR_VECTOR)
#endif
#define CP0SR_ENABLE_INTS	(CP0SR_DISABLE_INTS | CP0SR_HARD_INT4 | CP0SR_HARD_INT2)


/* Interrupt Status Register 1 at offset 100 */
#define INT1_CHIDMAHALF		BIT(29)
#define INT1_CHIDMAFULL		BIT(28)
#define INT1_SND1_0INT		BIT(21)
#define INT1_CHIRXAINT		BIT(26)
#define INT1_CHIRXBINT		BIT(25)
#define INT1_CHIERRINT		BIT(23)

/* Interrupt Status Register 2 at offset 104 */
#define INT2_UARTARXINT		BIT(31)
#define INT2_UARTARXOVERRUN	BIT(30)
#define INT2_UARTAFRAMEINT	BIT(29)
#define INT2_UARTABREAKINT	BIT(28)
#define INT2_UARTATXINT		BIT(26)
#define INT2_UARTATXOVERRUN	BIT(25)
#define INT2_UARTAEMPTY		BIT(24)

#define INT2_UARTBRXINT		BIT(21)
#define INT2_UARTBRXOVERRUN	BIT(20)
#define INT2_UARTBFRAMEINT	BIT(29)
#define INT2_UARTBBREAKINT	BIT(18)
#define INT2_UARTBTXINT		BIT(16)
#define INT2_UARTBTXOVERRUN	BIT(15)
#define INT2_UARTBEMPTY		BIT(14)

#define INT2_UARTA_RX		(BIT(31) | BIT(30) | BIT(29) | BIT(28) | BIT(27))
#define INT2_UARTA_TX		(BIT(26) | BIT(25) | BIT(24))
#define INT2_UARTA_DMA		(BIT(23) | BIT(22))

#define INT2_UARTB_RX		(BIT(21) | BIT(20) | BIT(19) | BIT(18) | BIT(17))
#define INT2_UARTB_TX		(BIT(16) | BIT(15) | BIT(14))
#define INT2_UARTB_DMA		(BIT(13) | BIT(12))

/* Interrupt Status Register 5 */
#define INT5_ALARMINT	 BIT(30)
#define INT5_PERIODICINT BIT(29)
#define INT5_POSPWRINT 	 BIT(27)
#define INT5_NEGPWRINT	 BIT(26)
#define INT5_POSPWROKINT BIT(25)
#define INT5_NEGPWROKINT BIT(24)
#define INT5_POSONBUTINT BIT(23)
#define INT5_NEGONBUTINT BIT(22)
#define INT5_SPIAVAILINT BIT(21)
#define INT5_SPIRCVINT	 BIT(19)
#define INT5_SPIEMPTYINT BIT(18)
#define INT5_IOPOSINT6	 BIT(13)
#define INT5_IOPOSINT5	 BIT(12)
#define INT5_IONEGINT6	 BIT(6)
#define INT5_IONEGINT5	 BIT(5)

#define INT5_NEGINT_SHIFT 0
#define INT5_POSINT_SHIFT 7

/* Interrupt Status Register 6 */
#define INT6_IRQHIGH	BIT(31)
#define INT6_IRQLOW	BIT(30)
#define INT6_INTVECT	(BIT(5) | BIT(4) | BIT(3) | BIT(2))


/* Interrupt Enable Register 6 */

#define INT6_GLOBALEN		BIT(18)
#define INT6_PWROKINT		BIT(15)
#define INT6_PERIODICINT 	BIT(13)
#define INT6_UARTARXINT		BIT(11)
#define INT6_UARTBRXINT		BIT(10)


/******************************************************************************
*
*	08  Timer and clock modules
*
******************************************************************************/

/* Register locations */

#define RTChigh	      	REG_AT(0x140)
#define RTClow	        REG_AT(0x144)
#define RTCalarmHigh    REG_AT(0x148)
#define RTCalarmLow     REG_AT(0x14c)
#define RTCtimerControl REG_AT(0x150)
#define RTCperiodTimer  REG_AT(0x154)
#define ClockControl  	REG_AT(0x1C0)



/* Timer control Register */
#define TIM_ENPERTIMER	BIT(4)
#define TIM_RTCCLEAR	BIT(3)

/* Periodic Timer Register */

/* For a system clock frequency of 36.864MHz, the timer counts at one tick
   every 868nS (ie CLK/32). Therefore 1152 counts gives a 1mS interval
 */

#define PER_TIMER_COUNT		1152
#define PER_TIMER_DIVISOR	  10

/* Clock Control Register */

#define CLK_CHICLKDIR	BIT(20)
#define CLK_ENCHIMCLK	BIT(19)
#define CLK_ENVIDCLK	BIT(18)
#define CLK_ENSPICLK	BIT(16)
#define CLK_ENTIMERCLK	BIT(15)
#define CLK_SIBMCLKDIR	BIT(13)
#define CLK_ENSIBMCLK	BIT(11)
#define CLK_CSERSEL	BIT(7)
#define CLK_ENCSERCLK	BIT(3)
#define CLK_EN_UART_A	BIT(1)
#define CLK_EN_UART_B	BIT(0)

/* Clock Control Register */

#define CLK_CHICLKDIV_MASK	(BIT(24) | BIT(25) | BIT(26) | BIT(27) | BIT(28) | BIT(29) | BIT(30) | BIT(31))
#define CLK_CHICLKDIV_SHIFT 24

#define CLK_CSERDIV_MASK	(BIT(4) | BIT(5) | BIT(6))
#define CLK_CSERDIV_SHIFT	4

#define CLK_SIBMCLK_MASK	(BIT(8) | BIT(9) | BIT(10))
#define CLK_SIBMCLK_SHIFT	8

/******************************************************************************
*
*	09  GPIO and MFIO modules
*
******************************************************************************/

#define GPIOcontrol   	REG_AT(0x180)

/******************************************************************************
*
*	12  Aliases for general processor registers
*
******************************************************************************/

#ifdef ASSEMBLY
/* config register */

#define CFG_FREQUENCY_MASK	(BIT(10) | BIT(11))
#define CFG_DOZE		BIT(9)
#define	CFG_HALT		BIT(8)
#define CFG_FREQUENCY_SHIFT 	10

#endif
